/*
 *  Purpose:
 *    This script provides the autoinc_gen procedure. It can be used to
 *    dynamically generate triggers that set generators.
 *
 *
 *  Usage:
 *    This script must be executed in ISQL for the Flyspray database
 *    before running setup
 *
 *	Warning:
 *		Using EXECUTE STATEMENT with DDL can be dangerous if COMMIT
 *		is not called immediately after calling the procedure that
 *		calls EXECUTE STATEMENT.
 */

--Dynamically create trigger that fires a generator on a primary key column
set term ^;
create procedure autoinc_gen (TABLE_NAME varchar(27), COLUMN_NAME varchar(31), SEQUENCE_NAME varchar(31) )
as
declare variable trig_name varchar(31);
declare variable sql varchar(1024);
begin

trig_name='tib_' || :TABLE_NAME;

sql = 'create trigger ' || :trig_name || ' FOR ' || :TABLE_NAME ||
    ' BEFORE INSERT POSITION 0 AS ' ||
	' BEGIN ' ||
	'   IF (( new.' || :COLUMN_NAME || ' is null ) or ( new.' || :COLUMN_NAME || '=0)) then ' ||
	'new.' || :COLUMN_NAME || '= gen_id(' || :SEQUENCE_NAME || ',1); ' ||
	' END'
	;

execute statement sql;

end^
set term ;^
commit;

